from configs.config import *

from torch.cuda import amp


def render_uncondition(conf: TrainConfig,
                       model: BeatGANsAutoencModel,
                       x_T,
                       sampler: Sampler,
                       latent_sampler: Sampler,
                       conds_mean=None,
                       conds_std=None,
                       clip_latent_noise: bool = False):
    device = x_T.device
    if conf.train_mode == TrainMode.diffusion:
        assert conf.model_type.can_sample()
        return sampler.sample(model=model, noise=x_T)
    elif conf.train_mode.is_latent_diffusion():
        model: BeatGANsAutoencModel
        if conf.train_mode == TrainMode.latent_diffusion:
            latent_noise = torch.randn(len(x_T), conf.style_ch, device=device)
        else:
            raise NotImplementedError()

        if clip_latent_noise:
            latent_noise = latent_noise.clip(-1, 1)

        cond = latent_sampler.sample(
            model=model.latent_net,
            noise=latent_noise,
            clip_denoised=conf.latent_clip_sample,
        )

        if conf.latent_znormalize:
            cond = cond * conds_std.to(device) + conds_mean.to(device)

        # the diffusion on the model
        return sampler.sample(model=model, noise=x_T, cond=cond)
    else:
        raise NotImplementedError()

def render_condition(
    conf: TrainConfig,
    model: BeatGANsAutoencModel,
    x_T,
    sampler: Sampler,
    x_start=None,
    cond=None,
    mask=None,
    cond_other=None,
    avg_score=None,
    postprocess_fn=None,
):
    if conf.train_mode == TrainMode.diffusion:
        assert conf.model_type.has_autoenc()
        # returns {'cond', 'cond2'}

        if cond is None:
            cond_dict = model.encode(x_start)
            cond = cond_dict['cond']
            cond_other = cond_dict['cond_other']

            if conf.data_name == 'vox256' and 'naive' in conf.name:
                cond = torch.mean(cond, dim=0, keepdim=True).expand(len(x_start), -1)   #EDIT
        #print(cond_other)
        return sampler.sample(model=model,
                              noise=x_T,
                              avg_score=avg_score,
                              model_kwargs={'cond': cond, 'cond_other': cond_other, 'mask': mask},
                              postprocess_fn=postprocess_fn)
    else:
        raise NotImplementedError()
    
def render_direction(
    conf: TrainConfig,
    model: BeatGANsAutoencModel,
    x_T,
    T,
    t,
    sampler: Sampler,
    x_start=None,
    cond=None,
    cond_other=None,
):
    if conf.train_mode == TrainMode.diffusion:
        assert conf.model_type.has_autoenc()
        # returns {'cond', 'cond2'}

        if cond is None:
            cond_dict = model.encode(x_start)
            cond = cond_dict['cond']
            cond_other = cond_dict['cond_other']

            if conf.data_name == 'vox256' and 'naive' in conf.name:
                cond = torch.mean(cond, dim=0, keepdim=True).expand(len(x_start), -1)   #EDIT
        #print(cond_other)
        return sampler.p_mean_similarity(model,
                              x_T,
                              t,
                              model_kwargs={'cond': cond, 'cond_other': cond_other})
    else:
        raise NotImplementedError()
